#include<limits>
#include<algorithm>
#include<forward_list>
#include"swr_object.hpp"
#include"swr_mesh.hpp"
#include"sdlut.hpp"


#ifdef EMSCRIPTEN
#include"emscripten.h"
#endif


sdlut::Screen
sdlscreen;


using namespace swr;


namespace{


constexpr int  screen_w = 480;
constexpr int  screen_h = 480;


std::forward_list<Point>
pbuf;


Object
world;


Object
seen_world;


Vector
light;


RotationParameter
view;


Screen
screen;


Mesh
mesh;


Object*
obj;


Point*
new_point(int  x, int  y, int  z)
{
  pbuf.emplace_front(x,y,z);

  return &pbuf.front();
}


Point  cursor_point;
Point  first_point(-1);


int
update_flag;


void
draw_rect(int  x, int  y)
{
  sdlscreen.draw_rect(0xFFFFFFFF,x,y,16,16);
}


void
update(bool  force=false)
{
    if(!update_flag && !force)
    {
      return;
    }


  sdlscreen.lock();

  seen_world = world.transform(nullptr,&view,&light);

  screen.clear();

  seen_world.render(screen);

    for(int  y = screen.get_height()-1;  y >= 0;  --y)
    {
      auto  row = sdlscreen.get_row(screen_h-1-y);

        for(int  x = 0;  x < screen.get_width();  ++x)
        {
          auto&  e = screen.get_element(x,y);

          auto  color = sdlscreen.map_rgb(e.color.r,e.color.g,e.color.b);

          row.put(color,x);
        }
    }


  draw_rect(cursor_point.x,cursor_point.y);

    if(first_point.x >= 0)
    {
      draw_rect(first_point.x-8,first_point.y-8);
    }


  update_flag = 0;


  sdlscreen.unlock();
  sdlscreen.update();
}


void
motion(const SDL_MouseMotionEvent&  evt)
{
  int  x = evt.x&0xFFF0;
  int  y = evt.y&0xFFF0;

  cursor_point.x = x-8;
  cursor_point.y = y-8;

  update_flag = true;
}


void
button(const SDL_MouseButtonEvent&  evt)
{
  constexpr int  z = -200;

  int  x = evt.x&0xFFF0;
  int  y = evt.y&0xFFF0;

    if(evt.button == SDL_BUTTON_LEFT)
    {
        if(first_point.x >= 0)
        {
          mesh.make_edge(new_point(first_point.x-screen_w/2,screen_h/2-first_point.y,z),
                         new_point(            x-screen_w/2,screen_h/2-            y,z),true);

          obj->lines.clear();


          mesh.convert_to_lines(Color(255,0,0),obj->lines);

//          printf("number of vertexes = %4d\n",mesh.get_vertex_array().size());
          printf("number of edges(by mesh  ) = %4d\n",mesh.get_matrix().get_count());
          printf("number of edges(by matrix) = %4d\n",mesh.get_edge_count());

          first_point.x = -1;
        }

      else
        {
          first_point.x = x;
          first_point.y = y;
        }
    }

  else
    if(evt.button == SDL_BUTTON_RIGHT)
    {
        if(first_point.x >= 0)
        {
          first_point.x = -1;
        }

      else
        if(mesh.cut_polygon(Color(255,0,0),obj->polygons))
        {
          obj->lines.clear();

          auto  n = mesh.get_vertex_count();

          printf("number of vertexes = %6d\n",n);
          printf("number of edges(by mesh  ) = %4d\n",mesh.get_matrix().get_count());
          printf("number of edges(by matrix) = %4d\n",mesh.get_edge_count());
        }
    }


  cursor_point.x = x-8;
  cursor_point.y = y-8;

  update_flag = true;
}


void
keydown(const SDL_KeyboardEvent&  evt)
{
    switch(evt.keysym.sym)
    {
      case(SDLK_LEFT ): obj->y_degree-=4;break;
      case(SDLK_RIGHT): obj->y_degree+=4;break;
      case(SDLK_UP   ): obj->x_degree+=4;break;
      case(SDLK_DOWN ): obj->x_degree-=4;break;
    }


  update_flag = true;
}


void
main_loop()
{
  static SDL_Event  evt;

    while(SDL_PollEvent(&evt))
    {
        switch(evt.type)
        {
          case(SDL_MOUSEBUTTONUP): button(evt.button);break;
          case(SDL_MOUSEMOTION): motion(evt.motion);break;
          case(SDL_KEYDOWN): keydown(evt.key);break;
          case(SDL_QUIT):
            sdlscreen.destroy();
            SDL_Quit();
            mesh.print();
            exit(0);
            break;
        }
    }


  update();
}


}


int
main(int  argc, char**  argv)
{
  SDL_Init(SDL_INIT_VIDEO);

  sdlscreen.create("swrtest - " __DATE__,screen_w,screen_h);

  screen.resize(screen_w,screen_h);

  light.x =1;
  light.y =-1;
  light.z=-1;

  obj = &world;

mesh.make_edge(new_point(-128,  96,-200),new_point( -32, 176,-200),true);
mesh.make_edge(new_point(-128,  96,-200),new_point(-144, -16,-200),true);
mesh.make_edge(new_point(-128,  96,-200),new_point( -16,  80,-200),true);
mesh.make_edge(new_point( -32, 176,-200),new_point(  64, 160,-200),true);
mesh.make_edge(new_point( -32, 176,-200),new_point( -16,  80,-200),true);
mesh.make_edge(new_point( -32, 176,-200),new_point(  32, 112,-200),true);
mesh.make_edge(new_point(  64, 160,-200),new_point( 144,  96,-200),true);
mesh.make_edge(new_point(  64, 160,-200),new_point(  32, 112,-200),true);
mesh.make_edge(new_point( 144,  96,-200),new_point( 128,  16,-200),true);
mesh.make_edge(new_point( 144,  96,-200),new_point(  32, 112,-200),true);
mesh.make_edge(new_point( 128,  16,-200),new_point(  48, -64,-200),true);
mesh.make_edge(new_point( 128,  16,-200),new_point(  32, 112,-200),true);
mesh.make_edge(new_point( 128,  16,-200),new_point(  32,  32,-200),true);
mesh.make_edge(new_point(  48, -64,-200),new_point( -80, -48,-200),true);
mesh.make_edge(new_point(  48, -64,-200),new_point(  32,  32,-200),true);
mesh.make_edge(new_point( -80, -48,-200),new_point(-144, -16,-200),true);
mesh.make_edge(new_point( -80, -48,-200),new_point( -16,  80,-200),true);
mesh.make_edge(new_point( -80, -48,-200),new_point(  32,  32,-200),true);
mesh.make_edge(new_point(-144, -16,-200),new_point( -16,  80,-200),true);
mesh.make_edge(new_point( -16,  80,-200),new_point(  32, 112,-200),true);
mesh.make_edge(new_point( -16,  80,-200),new_point(  32,  32,-200),true);
mesh.make_edge(new_point(  32, 112,-200),new_point(  32,  32,-200),true);

mesh.convert_to_lines(Color(255,0,0),obj->lines);

printf("%4d edges\n",mesh.get_matrix().get_count());

  update(true);

#ifdef EMSCRIPTEN
  emscripten_set_mmain_loop(main_loop,-1,false);
#else
    for(;;)
    {
      main_loop();

      SDL_Delay(20);
    }
#endif


  return 0;
}




