#include"swr_adjacencymatrix.hpp"
#include<cstdio>




namespace{


/*
1,2,4,7,B,
  3,5,8,C,
    6,9,D,
      A,E,
        F,
*/

std::vector<int>
base_indexes({0,1,3,6,10,15});


void
extend(int  end)
{
  int  v = base_indexes.back();

    while(v < end)
    {
      v += base_indexes.size();

      base_indexes.emplace_back(v);
    }
}


}


namespace swr{




AdjacencyMatrix::
AdjacencyMatrix():
size(0),
count(0)
{
}




AdjacencyMatrix&
AdjacencyMatrix::
operator=(const AdjacencyMatrix&  rhs)
{
  resize(rhs.size);

  buffer = rhs.buffer;
  count  = rhs.count;

  return *this;
}


void
AdjacencyMatrix::
clear()
{
  std::fill(buffer.begin(),buffer.end(),0);

  count = 0;
}


void
AdjacencyMatrix::
resize(int  size)
{
    if(size)
    {
      this->size = size;

      buffer.resize(size*(size-1)/2);

      extend(buffer.size());
    }
}


size_t
AdjacencyMatrix::
get_size() const
{
  return size;
}


size_t
AdjacencyMatrix::
get_count() const
{
  return count;
}




const uint8_t*
AdjacencyMatrix::
get_pointer(int  x, int  y) const
{
    if(x < y)
    {
//printf("%3d %3d -> ",x,y);
      std::swap(x,y);
//printf("%3d %3d\n",x,y);
    }


    if(!x || (x == y))
    {
      return nullptr;
    }


  return &buffer[base_indexes[x-1]+y];
}


bool
AdjacencyMatrix::
set_value(int  x, int  y, uint8_t  v)
{
  auto  ptr = const_cast<uint8_t*>(get_pointer(x,y));

    if(ptr)
    {
        if(v)
        {
            if(!*ptr)
            {
              *ptr = v;

              ++count;

              return true;
            }
        }

      else
        {
            if(*ptr)
            {
              *ptr = v;

              --count;

              return true;
            }
        }
    }


  return false;
}


uint8_t
AdjacencyMatrix::
get_value(int  x, int  y) const
{
  auto  p = get_pointer(x,y);

  return(p? *p:0);
}


void
AdjacencyMatrix::
print() const
{
  printf("matrix size = %8d\n",size);
  printf("buffer size = %8d\n",buffer.size());
  printf("true count = %8d\n",count);

/*
    for(int  y = 0;  y < size;  ++y)
    {
      const uint8_t*  p = &buffer[size*y];

        for(int  x = 0;  x < size;  ++x)
        {
          printf("%d,",*p++);
        }


      printf("\n");
    }
*/
}


void
AdjacencyMatrix::
print_index_table()
{
    for(auto  i: base_indexes)
    {
      printf("%8d\n",i);
    }
}




}




