#include"swr_bclinecontext.hpp"
#include<algorithm>


namespace swr{


BCLineContext::
BCLineContext(int  x0, int  y0, int  x1, int  y1,
              int  x2, int  y2, int  x3, int  y3)
{
  reset(x0,y0,x1,y1,
        x2,y2,x3,y3);
}




void
BCLineContext::
reset(int  x0, int  y0, int  x1, int  y1,
      int  x2, int  y2)
{
  p0.x  = x0;
  p0.y  = y0;
  p1.x  = x1;
  p1.y  = y1;
  p2.x  = x2;
  p2.y  = y2;

  t = 0.0;


  cp_number = 2;

  finished = 0;

  step(x0,y0);
  step(x1,y1);

  lnctx.reset(Point(x0,y0),Point(x1,y1));
}


void
BCLineContext::
reset(int  x0, int  y0, int  x1, int  y1,
      int  x2, int  y2, int  x3, int  y3)
{
  p0.x  = x0;
  p0.y  = y0;
  p1.x  = x1;
  p1.y  = y1;
  p2.x  = x2;
  p2.y  = y2;
  p3.x  = x3;
  p3.y  = y3;

  t = 0.0;


  cp_number = 3;

  finished = 0;

  step(x0,y0);
  step(x1,y1);

  lnctx.reset(Point(x0,y0),Point(x1,y1));
}


void
BCLineContext::
step(int&  x, int&  y)
{
  const auto  v = (1-t);

    switch(cp_number)
    {
      case(2):
        x = v*v*p0.x + 2*v*t*p1.x + t*t*p2.x;
        y = v*v*p0.y + 2*v*t*p1.y + t*t*p2.y;
        break;
      case(3):
        x = v*v*v*p0.x + 3*v*v*t*p1.x + 3*v*t*t*p2.x + t*t*t*p3.x;
        y = v*v*v*p0.y + 3*v*v*t*p1.y + 3*v*t*t*p2.y + t*t*t*p3.y;
        break;
    }


  t += 1.0/24;
}


bool
BCLineContext::
step()
{
    if(!finished)
    {
        if(!lnctx.step())
        {
            if(t >= 1.0)
            {
              finished = 1;

              return false;
            }


          int  x0 = lnctx.get_x();
          int  y0 = lnctx.get_y();
          int  x1;
          int  y1;

          step(x1,y1);

          lnctx.reset(Point(x0,y0),Point(x1,y1));
        }


      return true;
    }


  return false;
}


int  BCLineContext::get_x() const{return lnctx.get_x();}
int  BCLineContext::get_y() const{return lnctx.get_y();}




}




