#include"swr_linecontext.hpp"
#include<algorithm>
#include<cstdio>
#include<cstdlib>




namespace{


constexpr int  amount = 16;


template<typename  T>
T
longest_add_amount(T  n0, T  n1)
{
  return(n0 < n1? 1:-1);
}


template<typename  T>
T
nolongest_add_amount(T  nolongest, T  longest, bool  negative)
{
    if(!longest)
    {
      return 0;
    }


  T  t = ((nolongest<<amount)/longest);

  return negative? -t:t;
}


constexpr int  x_is_longest = 0;
constexpr int  y_is_longest = 1;
constexpr int  z_is_longest = 2;


template<typename  T>
int
compare(T  x, T  y, T  z)
{
    if(x >= y)
    {
           if(x >= z){return x_is_longest;}
      else           {return z_is_longest;}
    }

  else
    {
           if(y >= z){return y_is_longest;}
      else           {return z_is_longest;}
    }
}


}




namespace swr{


LineContext::
LineContext(const Point&  pt0, const Point&  pt1)
{
  reset(pt0,pt1);
}




void
LineContext::
reset(const Point&  pt0, const Point&  pt1)
{
  point0 = pt0;
  point1 = pt1;

  x.dist = std::abs(pt0.x-pt1.x);
  y.dist = std::abs(pt0.y-pt1.y);
  z.dist = std::abs(pt0.z-pt1.z);

    switch(compare(x.dist,y.dist,z.dist))
    {
      case(x_is_longest):
        x.add_amount =   longest_add_amount(pt0.x,pt1.x);
        y.add_amount = nolongest_add_amount(y.dist,x.dist,pt0.y > pt1.y);
        z.add_amount = nolongest_add_amount(z.dist,x.dist,pt0.z > pt1.z);
        x.shift_amount = 0;
        y.shift_amount = amount;
        z.shift_amount = amount;
        count = x.dist;
        break;
      case(y_is_longest):
        x.add_amount = nolongest_add_amount(x.dist,y.dist,pt0.x > pt1.x);
        y.add_amount =   longest_add_amount(pt0.y,pt1.y);
        z.add_amount = nolongest_add_amount(z.dist,y.dist,pt0.z > pt1.z);
        x.shift_amount = amount;
        y.shift_amount = 0;
        z.shift_amount = amount;
        count = y.dist;
        break;
      case(z_is_longest):
        x.add_amount = nolongest_add_amount(x.dist,z.dist,pt0.x > pt1.x);
        y.add_amount = nolongest_add_amount(y.dist,z.dist,pt0.y > pt1.y);
        z.add_amount =   longest_add_amount(pt0.z,pt1.z);
        x.shift_amount = amount;
        y.shift_amount = amount;
        z.shift_amount = 0;
        count = z.dist;
        break;
    }


  x.value = (pt0.x<<x.shift_amount);
  y.value = (pt0.y<<y.shift_amount);
  z.value = (pt0.z<<z.shift_amount);
}



bool
LineContext::
step()
{
    if(count)
    {
      --count;

      x.value += x.add_amount;
      y.value += y.add_amount;
      z.value += z.add_amount;

      return true;
    }


  return false;
}


int
LineContext::
get_count() const
{
  return count;
}


int  LineContext::get_x() const{return (x.value>>x.shift_amount);}
int  LineContext::get_y() const{return (y.value>>y.shift_amount);}
int  LineContext::get_z() const{return (z.value>>z.shift_amount);}


Point
LineContext::
get_point() const
{
  return Point(get_x(),
               get_y(),
               get_z());
}


void
LineContext::
print(const char*  prefix, const char*  suffix) const
{
    if(prefix)
    {
      printf("%s",prefix);
    }


  printf("X: %4d to %4d\n",point0.x,point1.x);
  printf("Y: %4d to %4d\n",point0.y,point1.y);
  printf("Z: %4d to %4d\n",point0.z,point1.z);

    if(suffix)
    {
      printf("%s",suffix);
    }
}




}




