#ifndef SWR_MESH_HPP_INCLUDED
#define SWR_MESH_HPP_INCLUDED


#include"swr_point.hpp"
#include"swr_line.hpp"
#include"swr_polygon.hpp"
#include"swr_adjacencymatrix.hpp"
#include"swr_vertex.hpp"
#include<list>
#include<vector>




namespace swr{




class
Mesh
{
public:
  using index_t = int32_t;

private:
  struct Edge{
    uint32_t      a;
    uint32_t      b;
    uint32_t  count;

    Edge(uint32_t  a_, uint32_t  b_):
    a(a_), b(b_), count(1){}

    bool  test(uint32_t  a_, uint32_t  b_) const{return((a == a_) && (b == b_));}
  };

  std::vector<index_t>  freed_indexes;

  std::vector<Vertex>  vertexes;

  std::list<Edge>  edge_list;

  uint32_t  vertex_count;

  AdjacencyMatrix  matrix;

public:
  Mesh();
  Mesh(const Mesh&  rhs);


  void  clear();

  Mesh&  operator=(const Mesh&  rhs);

  const AdjacencyMatrix&  get_matrix() const;

  uint32_t  get_vertex_count() const;
  uint32_t  get_edge_count() const;

  index_t  insert_vertex(Point*  pt, bool  check_repeat=false);
  index_t  find_vertex(const Point&  pt) const;

  int  make_edge(Point*  a, Point*  b, bool  check_repeat=false);
  int  make_edge(index_t  a, index_t  b, bool  check_repeat=false);

  bool  test_edge(index_t  a, index_t  b) const;


  std::vector<index_t>  collect_vertex_that_adjoins(index_t  i) const;
  int  count_triangle_that_includes(index_t  a_index, index_t  b_index, index_t*  t=nullptr) const;

  void  unrefer_vertex(index_t  i);

  void  remove_edge(index_t  a, index_t  b);

  void  remove(index_t  a, index_t  b, index_t  c);

  bool  cut_polygon(const Color&  color, std::list<Polygon>&  dst);

  void  convert_to_lines(const Color&  color, std::list<Line>&  lines) const;

  void  print() const;

};



}



#endif




