#include"swr_vector.hpp"
#include<cstdio>
#include<cmath>




namespace swr{


void
Vector::
reset(Element  x_, Element  y_, Element  z_)
{
  x = x_;
  y = y_;
  z = z_;
}


void
Vector::
reset(const Point&  pt0, const Point&  pt1)
{
  x = pt1.x-pt0.x;
  y = pt1.y-pt0.y;
  z = pt1.z-pt0.z;
}


Vector  Vector::operator+(const Vector&  rhs) const{return Vector(x+rhs.x,y+rhs.y,z+rhs.z);}
Vector  Vector::operator-(const Vector&  rhs) const{return Vector(x-rhs.x,y-rhs.y,z-rhs.z);}


Vector&
Vector::
operator+=(const Vector&  rhs)
{
  x += rhs.x;
  y += rhs.y;
  z += rhs.z;

  return *this;
}


Vector&
Vector::
operator-=(const Vector&  rhs)
{
  x -= rhs.x;
  y -= rhs.y;
  z -= rhs.z;

  return *this;
}


Vector
Vector::
cross_product(const Vector&  a, const Vector&  b)
{
  return Vector(a.y*b.z-a.z*b.y,
                a.z*b.x-a.x*b.z,
                a.x*b.y-a.y*b.x);
}


Vector::Element
Vector::
dot_product(const Vector&  a, const Vector&  b)
{
  return(a.x*b.x+
         a.y*b.y+
         a.z*b.z);
}


void
Vector::
normalize()
{
  auto  len = std::sqrt(x*x+y*y+z*z);

  x /= len;
  y /= len;
  z /= len;
}


void
Vector::
print() const
{
  printf("[print Vector] X:%4f, Y:%4f, Z:%4f\n",x,y,z);
}




}




