#include<limits>
#include<algorithm>
#include"swr_object.hpp"
#include"sdlut.hpp"


#ifdef EMSCRIPTEN
#include"emscripten.h"
#endif


sdlut::Screen
sdlscreen;


using namespace swr;


namespace{


constexpr int  screen_w = 480;
constexpr int  screen_h = 480;


int  looping;


Object
seen_world;


Object
world;


RotationParameter
view;


Screen
screen;


Vector
light;


Object*  obj0;
Object*  obj1;
Object*  obj2;


int
modified = 1;




void
update()
{
  sdlscreen.lock();

  seen_world.clear();

  seen_world = world.transform(nullptr,&view,&light);

  screen.clear();

  seen_world.render(screen);

    for(int  y = screen.get_height()-1;  y >= 0;  --y)
    {
      auto  row = sdlscreen.get_row(screen_h-1-y);

        for(int  x = 0;  x < screen.get_width();  ++x)
        {
          auto&  e = screen.get_element(x,y);

          auto  color = sdlscreen.map_rgb(e.color.r,e.color.g,e.color.b);

          row.put(color,x);
        }
    }


  sdlscreen.unlock();
  sdlscreen.update();
}


void
keydown(const SDL_KeyboardEvent&  evt)
{
    if(SDL_GetModState()&KMOD_SHIFT)
    {
        switch(evt.keysym.sym)
        {
          case(SDLK_LEFT ): view.center.x-=4;break;
          case(SDLK_RIGHT): view.center.x+=4;break;
          case(SDLK_UP   ): view.center.y+=4;break;
          case(SDLK_DOWN ): view.center.y-=4;break;
        }

    }

  else
    {
        switch(evt.keysym.sym)
        {
          case(SDLK_LEFT ): obj1->x_degree-=4;break;
          case(SDLK_RIGHT): obj1->x_degree+=4;break;
          case(SDLK_UP   ): obj1->z_degree+=4;break;
          case(SDLK_DOWN ): obj1->z_degree-=4;break;
        }
    }


  modified = 1;
}


void
make_trigonal_pyramid(Object&  obj, const Point&  base, int  x_width, int  y_width, int  z_width)
{
  obj.push_polygon(Color(255,0,0),Point(base.x        ,base.y        ,base.z),
                                  Point(base.x        ,base.y+y_width,base.z),
                                  Point(base.x+x_width,base.y        ,base.z));
  obj.push_polygon(Color(0,255,0),Point(base.x,base.y        ,base.z        ),
                                  Point(base.x,base.y+y_width,base.z        ),
                                  Point(base.x,base.y        ,base.z+z_width));
  obj.push_polygon(Color(0,0,255),Point(base.x,base.y+y_width,base.z        ),
                                  Point(base.x+x_width,base.y,base.z        ),
                                  Point(base.x        ,base.y,base.z+z_width));
  obj.push_polygon(Color(255,255,0),Point(base.x        ,base.y,base.z        ),
                                    Point(base.x+x_width,base.y,base.z        ),
                                    Point(base.x        ,base.y,base.z+z_width));
}


void
main_loop()
{
  static SDL_Event  evt;

    while(SDL_PollEvent(&evt))
    {
        switch(evt.type)
        {
          case(SDL_KEYDOWN): keydown(evt.key);break;
          case(SDL_QUIT):
            sdlscreen.destroy();
            SDL_Quit();
            exit(0);
            break;
        }
    }


    if(modified)
    {
      update();

      modified = 0;
    }
}


}




int
main(int  argc, char**  argv)
{
  SDL_Init(SDL_INIT_VIDEO);

  sdlscreen.create("swrtest - " __DATE__,screen_w,screen_h);

  looping = true;

  screen.resize(screen_w,screen_h);


  view.center.z +=300;

  constexpr int  v = 80;

  obj0 = &world.make_child();
  obj1 = &obj0->make_child();
  obj2 = &obj1->make_child();

  make_trigonal_pyramid(*obj0,Point(0,0,0),v,v,v);
  make_trigonal_pyramid(*obj1,Point(80,0,0),v,v,v);
  make_trigonal_pyramid(*obj2,Point(80,80,0),v,v,v);

  light.x =  1;
  light.y = -1;
  light.z = -1;

  update();

#ifdef EMSCRIPTEN
  emscripten_set_main_loop(main_loop,-1,false);
#else
    for(;;)
    {
      main_loop();

      SDL_Delay(20);
    }
#endif


  return 0;
}




